@@ -10,6 +10,8 @@ from i2.deco import (
 
 from i2.signatures import (
     Sig,
+    Param,
+    sort_params,
     call_forgivingly,
     call_somewhat_forgivingly,
     name_of_obj,
@@ -52,5 +54,6 @@ from i2.util import (
     lazyprop,
     frozendict,
     inject_method,
-    make_sentinel,
+    mk_sentinel,
+    ensure_identifiers,
 )
