@@ -146,18 +146,6 @@ def _multi_func_init(self, *unnamed_funcs, **named_funcs) -> None:
 _dflt_signature = Signature.from_callable(lambda *args, **kwargs: None)
 
 
-def _signature_from_first_and_last_func(first_func, last_func):
-    try:
-        input_params = signature(first_func).parameters.values()
-    except ValueError:  # function doesn't have a signature, so take default
-        input_params = _dflt_signature.parameters.values()
-    try:
-        return_annotation = signature(last_func).return_annotation
-    except ValueError:  # function doesn't have a signature, so take default
-        return_annotation = _dflt_signature.return_annotation
-    return Signature(input_params, return_annotation=return_annotation)
-
-
 class MultiObj(Mapping):
     """A base class that holds several named objects
 
@@ -266,6 +254,18 @@ class MultiFunc(MultiObj):
         iterable_of_callables_validation(self.funcs.values())
 
 
+def _signature_from_first_and_last_func(first_func, last_func):
+    try:
+        input_params = signature(first_func).parameters.values()
+    except ValueError:  # function doesn't have a signature, so take default
+        input_params = _dflt_signature.parameters.values()
+    try:
+        return_annotation = signature(last_func).return_annotation
+    except ValueError:  # function doesn't have a signature, so take default
+        return_annotation = _dflt_signature.return_annotation
+    return Signature(input_params, return_annotation=return_annotation)
+
+
 # TODO: Give it a __name__ and make it more like a "normal" function so it works
 #  well when so assumed?
 class Pipe(MultiFunc):
@@ -293,6 +293,18 @@ class Pipe(MultiFunc):
     Notes:
         - Pipe instances don't have a __name__ etc. So some expectations of normal functions are not met.
         - Pipe instance are pickalable (as long as the functions that compose them are)
+
+    You can specify a single functions:
+
+    >>> Pipe(lambda x: x + 1)(1)
+    2
+
+    but
+
+    >>> Pipe()
+    Traceback (most recent call last):
+      ...
+    ValueError: You need to specify at least one function!
     """
 
     def __init__(self, *unnamed_funcs, **named_funcs):
@@ -302,17 +314,19 @@ class Pipe(MultiFunc):
         # The extra initialization for pipelines
         callables = list(self.funcs.values())
         n_funcs = len(callables)
-        other_funcs = ()
         if n_funcs == 0:
             raise ValueError('You need to specify at least one function!')
+
         elif n_funcs == 1:
+            other_funcs = ()
             first_func = last_func = callables[0]
         else:
-            first_func, *other_funcs, last_func = callables
+            first_func, *other_funcs = callables
+            *_, last_func = other_funcs
 
         self.__signature__ = _signature_from_first_and_last_func(first_func, last_func)
         self.first_func = first_func
-        self.other_funcs = tuple(other_funcs) + (last_func,)
+        self.other_funcs = other_funcs
 
     def __call__(self, *args, **kwargs):
         out = self.first_func(*args, **kwargs)
@@ -320,6 +334,103 @@ class Pipe(MultiFunc):
             out = func(out)
         return out
 
+    def __len__(self):
+        return len(self.funcs)
+
+    def __eq__(self, other):
+        return pipes_are_equal(self, other)
+
+
+from operator import eq
+
+
+def _flatten_pipe(pipe):
+    for func in pipe.funcs.values():
+        if isinstance(func, Pipe):
+            yield from _flatten_pipe(func)
+        else:
+            yield func
+
+
+def flatten_pipe(pipe):
+    """Unravel nested Pipes to get a flat 'sequence of functions' version of input.
+
+    >>> def f(x): return x + 1
+    >>> def g(x): return x * 2
+    >>> def h(x): return x - 3
+    >>> a = Pipe(f, g, h)
+    >>> b = Pipe(f, Pipe(g, h))
+    >>> len(a)
+    3
+    >>> len(b)
+    2
+    >>> c = flatten_pipe(b)
+    >>> len(c)
+    3
+    >>> assert a(10) == b(10) == c(10) == 19
+    """
+    return Pipe(*_flatten_pipe(pipe))
+
+
+def pipes_are_equal(p1, p2, *, func_equality=eq, verbose=False):
+    """Determine if two pipelines are equal.
+
+    Pipelines are equal if their flattened versions have equal functions.
+    Function equality can be controlled by the ``func_equality`` argument.
+    The ``verbose`` argument will print some more information about why the pipelines
+    are not equal.
+
+    >>> def f(x): return x + 1
+    >>> def g(x): return x * 2
+    >>> def h(x): return x - 3
+    >>> a = Pipe(f, g, h)
+    >>> b = Pipe(f, g, h)
+    >>> c = Pipe(f, Pipe(g, h))
+    >>> assert a(10) == b(10) == c(10) == 19
+    >>> pipes_are_equal(a, b)
+    True
+    >>> pipes_are_equal(a, c)
+    True
+    >>> pipes_are_equal(Pipe(f, g), Pipe(g, h))
+    False
+    >>> pipes_are_equal(Pipe(f, g), Pipe(f, g, h))
+    False
+
+    Get more information when pipes are not equal.
+
+    >>> pipes_are_equal(Pipe(f, g), Pipe(f, g, h), verbose=True)
+    --> Flattened pipes do not have the same number of functions: len(p1)=2 != len(p2)=3
+    False
+
+    Change how functions are compared for equality:
+
+    >>> pipes_are_equal(Pipe(lambda x: x), Pipe(lambda x: x))
+    False
+    >>> from inspect import getsource
+    >>> source_equality = lambda f, ff: getsource(f) == getsource(ff)
+    >>> pipes_are_equal(
+    ...     Pipe(lambda x: x), Pipe(lambda x: x), func_equality=source_equality
+    ... )
+    True
+
+    """
+    p1, p2 = map(flatten_pipe, (p1, p2))
+    if len(p1) != len(p2):
+        if verbose:
+            print(
+                f'--> Flattened pipes do not have the same number of functions:'
+                f' {len(p1)=} != {len(p2)=}'
+            )
+        return False  # if not same number of keys or keys different, not equality
+    else:
+        for func1, func2 in zip(p1.funcs.values(), p2.funcs.values()):
+            if not func_equality(func1, func2):
+                if verbose:
+                    print(f'--> func1 and func2 are not equal: {func1=} != {func2=}')
+                return False  # these two funcs are not equal
+            # else continue
+    return True
+
 
 class FuncFanout(MultiFunc):
     """Applies multiple functions to the same argument(s) and returns a dict of results.
@@ -428,7 +539,7 @@ class FlexFuncFanout(MultiFunc):
 
     >>> from inspect import signature
     >>> signature(mf1)
-    <Signature (w, x: float, a, y=1, z: int = 1, b: float = 0.0)>
+    <Sig (w, x: float, a, y=1, z: int = 1, b: float = 0.0)>
 
     >>> mf2 = FlexFuncFanout(formula1, mult, addition=add, mysum=sum_of_args)
     >>> assert mf2.kwargs_for_func(
@@ -437,13 +548,14 @@ class FlexFuncFanout(MultiFunc):
     ... 'formula1': {'w': 1, 'x': 2, 'z': 3},
     ... 'mult': {'x': 2},
     ... 'addition': {'a': 4, 'b': 5},
-    ... 'mysum': {
-    ...     'kwargs': {
-    ...         'w': 1, 'x': 2, 'z': 3, 'a': 4, 'b': 5,
-    ...         'args': (7, 8), 'kwargs': {'a': 42}, 'extra_stuff': 'ignore'
-    ...         }
-    ...     }
-    ... }
+    ... 'mysum':
+    ...     {'kwargs': {'w': 1,
+    ...                 'x': 2,
+    ...                 'z': 3,
+    ...                 'a': 42,
+    ...                 'b': 5,
+    ...                 'args': (7, 8),
+    ...                 'extra_stuff': 'ignore'}}}
 
     """
 
