@@ -150,7 +150,7 @@ def mk_func_from_params(
     >>> str(Sig(f))
     '(a00, a01, /, a12, a13, a14, *a25, a36, **a47)'
     >>> f(0, 1, 2, 3, 4, 5, -5, a36=6, a47={"a47": 7, "a47_": -7})
-    "a00=0, a01=1, a12=2, a13=3, a14=4, a25=(5, -5), a36=6, a47={'a47': {'a47': 7, 'a47_': -7}}"
+    "a00=0, a01=1, a12=2, a13=3, a14=4, a25=(5, -5), a36=6, a47={'a47': 7, 'a47_': -7}"
     >>> f(0, 1, 2, a13=3, a14=4, a36=6)
     'a00=0, a01=1, a12=2, a13=3, a14=4, a25=(), a36=6, a47={}'
 
