@@ -3,9 +3,136 @@ from operator import attrgetter, itemgetter
 import inspect
 import re
 import itertools
+import sys
 import functools
 import types
-from typing import Mapping, Iterable
+from typing import (
+    Mapping,
+    Callable,
+    Any,
+    MutableMapping,
+    Union,
+    Optional,
+    Iterable,
+    TypeVar,
+)
+
+T = TypeVar('T')
+
+
+class OverwritesForbidden(ValueError):
+    """Raise when a user is not allowed to overwrite a mapping's key"""
+
+
+def ensure_identifiers(
+    *objs: Iterable[T],
+    get_identfiers: Callable[[T], Iterable[str]] = str.split,
+    is_identifier: Callable[[str], bool] = str.isidentifier,
+):
+    """Ensure an iterable of identifiers
+
+    >>> list(ensure_identifiers('these', 'are', 'valid', 'identifiers'))
+    ['these', 'are', 'valid', 'identifiers']
+
+    By default, ``ensure_identifiers`` will apply ``str.split`` to each ``obj`` of
+    ``objs`` (assumed to be strings!) so that it can extract identifiers from
+    space-separated strings of identifiers:
+
+    >>> list(ensure_identifiers('these are valid identifiers'))
+    ['these', 'are', 'valid', 'identifiers']
+
+    You can control this functionality through the ``get_identfiers`` argument, for
+    example, disallowing such splitting, or enabling the extraction of identifiers
+    from other objects than strings.
+
+    >>> list(ensure_identifiers(
+    ...     {'this': 0, 'works': 1}, {'too': 2},
+    ...     get_identfiers=list
+    ... ))
+    ['this', 'works', 'too']
+
+    You can also control the ``is_identifier`` validatation function:
+
+    >>> def less_than_6_chars(s): return len(s) < 6
+    >>> list(ensure_identifiers('okay', 'too_long', is_identifier=less_than_6_chars))
+    Traceback (most recent call last):
+      ...
+    ValueError: too_long isn't an identifier according toless_than_6_chars
+
+    """
+    for obj in objs:
+        for identifier in get_identfiers(obj):
+            if is_identifier(identifier):
+                yield identifier
+            else:
+                raise ValueError(
+                    f"{identifier} isn't an identifier according to"
+                    f"{getattr(is_identifier, '__name__', str(is_identifier))}"
+                )
+
+
+def insert_name_based_objects_in_scope(
+    *names,
+    factory: Callable[[str], Any],
+    scope: MutableMapping,
+    allow_overwrites: bool = False,
+):
+    """
+    Make several string-parametrized objects and insert them in a scope (e.g. locals()).
+
+    This is useful when to avoid (error-prone) situations where we want the name we
+    assign an object to, to be aligned with it's internal name, such as::
+
+        foo = Factory('foo', ...)
+        bar = Factory('bar', ...)
+        baz = Factory('baz', ...)
+
+    :param names: Identifier (valid python variable name) strings.
+        These are used both as arguments of the ``factory`` and as keys for the
+        ``scope`` the object the factory makes will be inserted under.
+    :param factory: A function that takes a (valid python identifier) string and
+        returns an object parametrized by that string.
+    :param scope: The ``MutableMapping`` we want to insert the objects in.
+    :param allow_overwrites: Whether the objects we create can overwrite existing
+        objects the ``scope`` may already have. If we don't allow overwrites and we
+        try to write under an existing key, a ``OverwritesForbidden`` error will be
+        raised. This also includes the situation where we have some duplicates in
+        ``names``.
+    :return: None (this function has the side effect of inserting items in ``scope``.
+
+    One of the (controversal) uses of ``insert_name_based_objects_in_scope`` is to be
+    able to make several string-parametrized
+
+    >>> from collections import namedtuple
+    >>> from functools import partial
+    >>>
+    >>> factory = partial(namedtuple, field_names='apple banana')
+    >>> insert_namedtuples_in_locals = partial(insert_name_based_objects_in_scope,
+    ...     factory=factory, scope=locals(), allow_overwrites=True
+    ... )
+    >>> insert_namedtuples_in_locals('foo bar', 'baz')
+
+    And now ``foo` exists!
+
+    >>> 'foo' in locals()
+    True
+    >>> foo(1,2)
+    foo(apple=1, banana=2)
+
+    And so does ``bar`` and ``baz``:
+
+    >>> bar(3, banana=4)
+    bar(apple=3, banana=4)
+    >>> baz(apple=3, banana=4)
+    baz(apple=3, banana=4)
+    """
+    for name in ensure_identifiers(*names):
+        if name not in scope or allow_overwrites:
+            scope[name] = factory(name)
+        else:
+            raise OverwritesForbidden(
+                f'This key already exisited and is not allowed to be overwritten'
+            )
 
 
 class Literal:
@@ -341,56 +468,158 @@ class MissingArgument(ValueError):
     pass
 
 
-def make_sentinel(name='_MISSING', var_name=None):
-    """Creates and returns a new **instance** of a new class, suitable for
-    usage as a "sentinel", a kind of singleton often used to indicate
-    a value is missing when ``None`` is a valid input.
+def _default_sentinel_repr_method(self):
+    return '%s(%r)' % (self.__class__.__name__, self.__name__)
 
-    Args:
-        name (str): Name of the Sentinel
-        var_name (str): Set this name to the name of the variable in
-            its respective module enable pickleability.
 
-    >>> make_sentinel(var_name='_MISSING')
-    _MISSING
+def mk_sentinel(
+    name,
+    boolean_value: bool = False,
+    repr_: Union[str, Callable] = _default_sentinel_repr_method,
+    *,
+    module: Optional[str] = None,
+):
+    """Creates and returns a new **instance** of a new class, suitable for usage as a
+    "sentinel" since it is a kind of singleton (there can be only one instance of it.)
 
-    The most common use cases here in boltons are as default values
-    for optional function arguments, partly because of its
-    less-confusing appearance in automatically generated
-    documentation. Sentinels also function well as placeholders in queues
-    and linked lists.
+    A frequent use case for sentinels are where we want to indicate that something is
+    missing. Often, we use ``None`` for this, but sometimes ``None`` is a valid value in
+    our context (see for example the ``inspect.Parameter.empty`` sentinel to indicate
+    that an argument doesn't have a default or annotation).
+    Other times, we may want to distinguish different kinds of "nothing".
 
-    .. note::
+    ``mk_sentinel`` can help you create such sentinels, takes care of annoying details
+    like pickability and allows you to control how to resolve your sentinel to a boolean.
 
-      By design, additional calls to ``make_sentinel`` with the same
-      values will not produce equivalent objects.
+    :param name: The name of your sentinel. Will be used for ``__name__`` attribute.
+    :param boolean_value: The boolean value that the sentinel instance should resolve to.
+    :param repr_: The method or string that should be used for the repr.
+    :param module:
+    :return: A sentinel instance
 
-      >>> make_sentinel('TEST') == make_sentinel('TEST')
-      False
-      >>> type(make_sentinel('TEST')) == type(make_sentinel('TEST'))
-      False
+    >>> Empty = mk_sentinel('Empty')
+    >>> Empty
+    Sentinel('Empty')
+
+    By default, the boolean resolution of a sentinel is ``False``. Meaning:
+
+    >>> Nothing = mk_sentinel('Nothing')
+    >>> bool(Nothing)
+    False
+
+    This is consistent with ``None``, so that you can check that an object ``x`` is not
+    ``Nothing`` by doing ``if x: ...`` or idioms like:
+
+    >>> x = Nothing
+    >>> x = x or 'default'
+    >>> x
+    'default'
+
+    (Though note that in situations where other elements that cast to ``False`` are
+    valid values for ``x`` (like ``0``, ``None``, or ``False`` itself), it's safer to use
+    ``if x is not Nothing: ...``.)
+
+    Anyway, I digress.
+    Point is that in some situations, the semantics  or usage of your sentinel is better
+    align with True. You can control what the boolean resolution of your
+    sentinel should be through the ``boolean_value`` argument:
+
+    >>> Empty = mk_sentinel('Empty', boolean_value=True)
+    >>> bool(Empty)
+    True
+
+    You can also control what you see in the repr, specifying a string value;
+
+    >>> Empty = mk_sentinel('undefined', repr_='undefined')
+    >>> Empty
+    undefined
+
+    or a method;
+
+    >>> Empty = mk_sentinel('Empty', repr_=lambda self: f"<{self.__name__}>")
+    >>> Empty
+    <Empty>
+
+    And yes, even though we used a lambda here, it's still picklable:
+
+    >>> import pickle
+
+    >>> Empty = mk_sentinel('Empty', repr_='Empty', module=__name__)
+    >>> pickle.loads(pickle.dumps(Empty))  # doctest: +SKIP
+    Empty
+
+    Talking about pickle, here's some more info on that:
+
+    >>> unpickled_Empty = pickle.loads(pickle.dumps(Empty))  # doctest: +SKIP
+    >>> # The unpickled version is "equal" to the original:
+    >>> unpickled_Empty == Empty  # doctest: +SKIP
+    True
+    >>> # the types are the same too:
+    >>> type(unpickled_Empty) == type(Empty)    # doctest: +SKIP
+    True
+    >>>
+    >>>
+
+    Note that though two sentinels might have the same name, they're not equal:
+
+    >>> Empty = mk_sentinel('Empty')
+    >>> AnotherEmptyWithSameName = mk_sentinel('Empty')
+    >>> Empty
+    Sentinel('Empty')
+    >>> AnotherEmptyWithSameName
+    Sentinel('Empty')
+    >>> # but...
+    >>> AnotherEmptyWithSameName == Empty
+    False
+    >>> # Note even the types are the same!
+    >>> type(AnotherEmptyWithSameName) == type(Empty)
+    False
+
+    One thing that makes the pickle work is that we took care of sticking in a
+    ``__module__`` for you. ``mk_sentinel`` figures this out by some dark magic
+    involving looking into the system's "frames" etc. This may not always work since
+    some systems (e.g. ``pypy``) may use different "under-the-hood" methods.
+
+    But if you want to control the value of ``__module__`` yourself, you can, simply
+    but indicating what the module of the sentinel is.
+    Usually, you'll just specify it as ``module=__name__``, which will stick the
+    name of the module you're defining the sentinel in for you!
+
+    >>> MySentinel = mk_sentinel('MySentinel', module=__name__)
+
+    Thanks: Inspired greately from the ``make_sentinel`` function of ``boltons``:
+    See https://boltons.readthedocs.io/.
 
     """
 
     class Sentinel(object):
         def __init__(self):
-            self.name = name
-            self.var_name = var_name
+            self.__name__ = name
 
-        def __repr__(self):
-            if self.var_name:
-                return self.var_name
-            return '%s(%r)' % (self.__class__.__name__, self.name)
+        if callable(repr_):
+            __repr__ = repr_
+        else:
 
-        if var_name:
+            def __repr__(self):
+                return repr_
 
-            def __reduce__(self):
-                return self.var_name
+        def __reduce__(self):
+            return self.__name__
 
-        def __nonzero__(self):
-            return False
+        def __bool__(self):
+            return boolean_value
 
-        __bool__ = __nonzero__
+    if module is None:
+        # TODO: Try to use something else than hidden _getframe
+        # TODO: extract this module resolver so can be reused (_getframe(2)?)
+        frame = sys._getframe(1)
+        module = frame.f_globals.get('__name__')
+
+    if not module or module not in sys.modules:
+        raise ValueError(
+            'Pickleable sentinel objects can only be made from top-level module scopes'
+        )
+    Sentinel.__module__ = module
 
     return Sentinel()
 
@@ -403,7 +632,7 @@ def _indent(text, margin, newline='\n', key=bool):
     return newline.join(indented_lines)
 
 
-NO_DEFAULT = make_sentinel(var_name='NO_DEFAULT')
+NO_DEFAULT = mk_sentinel('NO_DEFAULT', boolean_value=False)
 
 
 from inspect import formatannotation
