@@ -79,7 +79,7 @@ from typing import (
 
 from types import MethodType
 
-from i2.signatures import Sig, name_of_obj
+from i2.signatures import Sig, name_of_obj, KO, PK, VK
 from i2.multi_object import Pipe
 from i2.deco import double_up_as_factory
 
@@ -982,9 +982,33 @@ def mk_ingress_from_name_mapper(func, name_mapper: Mapping, *, conserve_kind=Fal
 
 # TODO: Replace with i2.wrapper instead
 # TODO: Make sure VARIADICs are handled properly, or error raised if present
-def nice_kinds(func):
-    """Wraps the func so it will only have POSITIONAL_OR_KEYWORD argument kinds.
 
+
+def apply_func_on_cond(func, cond, k, v):
+    if cond(k, v):
+        return func(v)
+    else:
+        return v
+
+
+def modify_dict_on_cond(d, cond, func):
+    return {k: apply_func_on_cond(func, cond, k, v) for k, v in d.items()}
+
+
+def convert_to_PK(kinds):
+    return {name: PK for name in kinds}
+
+
+def convert_VK_to_KO(kinds):
+    cond = lambda k, v: v == VK
+    func = lambda v: KO
+
+    return modify_dict_on_cond(kinds, cond, func)
+
+
+def nice_kinds(func, kinds_modifier=convert_to_PK):
+    """Wraps the func, changing the argument kinds according to kinds_modifier.
+    The default behaviour is to change all kinds to POSITIONAL_OR_KEYWORD kinds.
     The original purpose of this function is to remove argument-kind restriction
     annoyances when doing functional manipulations such as:
 
@@ -1011,7 +1035,11 @@ def nice_kinds(func):
     from i2 import Sig, call_somewhat_forgivingly
 
     sig = Sig(func)
-    sig = sig.ch_kinds(**{name: Sig.POSITIONAL_OR_KEYWORD for name in sig.names})
+    kinds_modif = kinds_modifier(sig.kinds)
+
+    sig = sig.ch_kinds(**kinds_modif)
+
+    # sig = sig.ch_kinds(**{name: Sig.POSITIONAL_OR_KEYWORD for name in sig.names})
 
     @wraps(func)
     def _func(*args, **kwargs):
@@ -1021,6 +1049,18 @@ def nice_kinds(func):
     return _func
 
 
+def wrap_from_sig(func, new_sig):
+    from i2 import call_somewhat_forgivingly
+
+    @wraps(func)
+    def _func(*args, **kwargs):
+        return call_somewhat_forgivingly(func, args, kwargs, enforce_sig=new_sig)
+
+    _func.__signature__ = new_sig
+
+    return _func
+
+
 # ---------------------------------------------------------------------------------------
 # wrap tools
 
